﻿//-----------------------------------------------------------------------
// <copyright file="PhotoService.cs" company="Sondre Bjellås">
// This software is licensed as Microsoft Public License (Ms-PL).
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FlickrNet;

namespace FlickrDownloadr.Services
{
    /// <summary>
    /// Represents an instance of any type of photo service that the UI can act upon.
    /// </summary>
    public class PhotoService : IPhotoService
    {
        private Flickr _flickr;

        public PhotoService(Flickr flickr)
        {
            _flickr = flickr;

            
        }

        //public Flickr Service { get; set; }
        
        public SearchOptions SearchOptions { get; set; }
        public Photo[] SelectedPhotos { get; set; }
        public PhotoSearchOptions Options { get; set; }

        public void Reset()
        {
            SearchOptions = new SearchOptions();
            Options = new PhotoSearchOptions();
            //AddLicenses(Options);
        }

        public void ResetAuthentication()
        {
            UserId = string.Empty;
            Username = string.Empty;
            IsAuthenticated = false;
            _flickr.AuthToken = string.Empty;

            Properties.Settings.Default.FlickrAuthorizationKey = "";
            Properties.Settings.Default.Save();
        }

        public void PersistToken(string token)
        {
            _flickr.AuthToken = token;

            FlickrDownloadr.Properties.Settings.Default.FlickrAuthorizationKey = token;
            FlickrDownloadr.Properties.Settings.Default.Save();
        }

        public Auth Authorize()
        {
            var auth = _flickr.AuthCheckToken(Properties.Settings.Default.FlickrAuthorizationKey);
            //labelStatus.Text = string.Format("You are authenticated as {0}.", auth.User.Username);

            UserId = auth.User.UserId;
            Username = auth.User.UserName;
            IsAuthenticated = true;

            _flickr.AuthToken = auth.Token;

            return auth;
        }
 
        public bool HasToken
        {
            get { return !string.IsNullOrEmpty(Properties.Settings.Default.FlickrAuthorizationKey); }
        }

        public bool IsAuthenticated { get; set; }

        public string Username { get; set; }
        public string UserId { get; set; }

        /// <summary>
        /// Finds the user id based upon username or email. Returns null if not found.
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        public FoundUser FindUserId(string userName)
        {
            FoundUser user = null;

            try
            {
                user = _flickr.PeopleFindByUserName(userName);
            }
            catch (FlickrNet.FlickrApiException ex1)
            {
                int code = ex1.Code;

                if (code == 1)
                {
                    try
                    {
                        user = _flickr.PeopleFindByEmail(userName);
                    }
                    catch (FlickrNet.FlickrApiException ex2)
                    {
                        // Swallow and return null
                    }
                }
            }

            return user;
        }

        private PhotoSearchOptions CreateOptions()
        {
            var options = new PhotoSearchOptions();
            options.SafeSearch = SafetyLevel.None;
            return options;
        }

        //private PhotoSearchOptions GetOptionsBasedOnCurrentSearch()
        //{
        //    PhotoSearchOptions options = CreateOptions();

        //    switch (SearchOptions.Type)
        //    {
        //        case SearchType.Tag:
        //            options.Tags = SearchOptions.Text;
        //            break;
        //        case SearchType.Search:
        //            options.Text = SearchOptions.Text;
        //            break;
        //        case SearchType.Group:
        //            options.GroupId = SearchOptions.Text;
        //            break;
        //        default:
        //            options = new PhotoSearchOptions(SearchOptions.UserId);
        //            break;
        //    }

        //    return options;
        //}

        public void AddLicenses(PhotoSearchOptions options, bool creativeCommonsOnly)
        {
            //if (Program.Debug)
            //{
            //    options.AddLicense(0);
            //}

            //options.Licenses.AddLicense(1);
            //options.AddLicense(2);
            //options.AddLicense(3);
            //options.AddLicense(4);
            //options.AddLicense(5);
            //options.AddLicense(6);

            options.Licenses.Clear();

            if (!creativeCommonsOnly)
            {
                options.Licenses.Add(LicenseType.AllRightsReserved);
            }

            options.Licenses.Add(LicenseType.AttributionCC);
            options.Licenses.Add(LicenseType.AttributionNoDerivativesCC);
            options.Licenses.Add(LicenseType.AttributionNoncommercialCC);
            options.Licenses.Add(LicenseType.AttributionNoncommercialNoDerivativesCC);
            options.Licenses.Add(LicenseType.AttributionNoncommercialShareAlikeCC);
            options.Licenses.Add(LicenseType.AttributionShareAlikeCC);
            options.Licenses.Add(LicenseType.NoKnownCopyrightRestrictions);
        }

        /// <summary>
        /// Returns all photos based on the current search terms, also photos licensed with "All Rights Reserved".
        /// </summary>
        /// <param name="page"></param>
        /// <param name="perPage"></param>
        /// <returns></returns>
        public PhotoCollection Preview(int page, int perPage, bool creativeCommonsOnly)
        {
            AddLicenses(Options, creativeCommonsOnly);

            var options = new PhotoSearchOptions();
            options.Page = page;
            options.PerPage = perPage;
            options.Extras = PhotoSearchExtras.All;

            return _flickr.PhotosSearch(options);
        }

        /// <summary>
        /// Returns only photos that users are allowed to download and is licensed under Creative Commons.
        /// </summary>
        /// <param name="page"></param>
        /// <param name="perPage"></param>
        /// <returns></returns>
        public void Search(int page, int perPage, bool creativeCommonsOnly, Action<FlickrResult<PhotoCollection>> callback)
        {
            var options = new PhotoSearchOptions();
            options.Page = page;
            options.PerPage = perPage;
            options.Extras = PhotoSearchExtras.All;

            AddLicenses(options, creativeCommonsOnly);

            _flickr.PhotosSearchAsync(options, callback);

            //var collection = _flickr.PhotosSearch(options);
            //return collection;
        }

        public void Search(int page, Action<FlickrResult<PhotoCollection>> callback)
        {
            Search(page, 20, callback);
        }

        public void Search(int page, int perPage, Action<FlickrResult<PhotoCollection>> callback)
        {
            Search(page, perPage, false, callback);
        }

        public void Search(PhotoSearchOptions searchOptions, Action<FlickrResult<PhotoCollection>> callback)
        {
            AddLicenses(searchOptions, false);
            _flickr.PhotosSearchAsync(searchOptions, callback);
        }
    }
}
