//-----------------------------------------------------------------------
// <copyright file="FormFlickr.cs" company="Sondre Bjells">
// This software is licensed as Microsoft Public License (Ms-PL).
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Microsoft.WindowsAPICodePack.Taskbar;
using FlickrDownloadr.UI;
using FlickrNet;
using FlickrDownloadr.Services;
using Flickr2Mesh;

namespace FlickrDownloadr
{
  public partial class FormFlickr : Form
  {
    // margins for API call used in Paint
    private VistaApi.Margins marg;

    // Rectangles; one for each edge of the form
    private Rectangle topRect = Rectangle.Empty;

    private Rectangle botRect = Rectangle.Empty;

    private Rectangle lefRect = Rectangle.Empty;

    private Rectangle rigRect = Rectangle.Empty;

    private List<UserControl> _navigationControls = new List<UserControl>();

    private int _navigationIndex = -1;

    public string Title { get; set; }

    public string Description { get; set; }


    private ThumbnailToolbarButton _stop;
    private ThumbnailToolbarButton _play;

    
    public FormFlickr()
    {
      InitializeComponent();

      this.Shown += new EventHandler(FormFlickr_Shown);

      this.SetStyle(ControlStyles.ResizeRedraw, true);

      this.FitGlass();
    }

    void FormFlickr_Shown(object sender, EventArgs e)
    {
        if (!Program.IsWindows7())
        {
            return;
        }

        // Windows 7: Create buttons for the new thumbnail preview.
        _stop = new ThumbnailToolbarButton(Properties.Resources.Stop1NormalBlue, "Cancel Downloading");
        _play = new ThumbnailToolbarButton(Properties.Resources.StepForwardNormalBlue, "Pause/Continue Downloading");

        DisableThumbnailToolbarButtons();

        _stop.Click += new EventHandler<ThumbnailButtonClickedEventArgs>(Stop_Click);
        _play.Click += new EventHandler<ThumbnailButtonClickedEventArgs>(Play_Click);

        TaskbarManager.Instance.ThumbnailToolbars.AddButtons(this.Handle, _play, _stop);
    }

    void Play_Click(object sender, ThumbnailButtonClickedEventArgs e)
    {
        if (panelContainer.Controls[0] is DownloadProgress)
        {
            DownloadProgress progressPanel = (DownloadProgress)panelContainer.Controls[0];
            progressPanel.PauseDownload();
        }
    }

    public void EnableThumbnailToolbarButtons()
    {
        _stop.Enabled = true;
        _play.Enabled = true;
    }

    public void DisableThumbnailToolbarButtons()
    {
        _stop.Enabled = false;
        _play.Enabled = false;
    }

    void Stop_Click(object sender, ThumbnailButtonClickedEventArgs e)
    {
        if (panelContainer.Controls[0] is DownloadProgress)
        {
            DownloadProgress progressPanel = (DownloadProgress)panelContainer.Controls[0];
            progressPanel.CancelDownload();
        }
    }

    private void FitGlass()
    {

        this.Paint += new PaintEventHandler(this.Form1_Paint);

      // If DWM is not enabled then get out
      if (!this.IsGlassEnabled())
      {
        return;
      }

      // Set the Margins to their default values
      marg.Top = panelContainer.Top; // extend from the top
      marg.Left = 0;  // not used in this sample but could be
      marg.Right = 0; // not used in this sample but could be
      marg.Bottom = 0;// not used in this sample but could be

      // call the function that gives us glass, 
      // passing a reference to our inset Margins
      VistaApi.DwmExtendFrameIntoClientArea(this.Handle, ref marg);      
    }



    private bool IsGlassEnabled()
    {
      if (Environment.OSVersion.Version.Major < 6)
      {
        return false;
      }

      //Check if DWM is enabled
      bool isGlassSupported = false;
      VistaApi.DwmIsCompositionEnabled(ref isGlassSupported);
      return isGlassSupported;
    }


    // Alpha-blending Paint after the glass extension
    // this seems better than the winforms transparency approach because here we can click on the glass!
    private void Form1_Paint(object sender, PaintEventArgs e)
    {
        Color color = Color.Black;

        if (WindowState == FormWindowState.Maximized)
        {
            color = Color.White;
        }

        // black brush for Alpha transparency
        SolidBrush blackBrush = new SolidBrush(Color.Black);

        Graphics g = e.Graphics;

        if (this.IsGlassEnabled())
        {      
            // setup the rectangles
            topRect = new Rectangle(0, 0, this.ClientSize.Width, marg.Top);
            lefRect = new Rectangle(0, 0, marg.Left, this.ClientSize.Height);
            rigRect = new Rectangle(this.ClientSize.Width - marg.Right, 0, marg.Right, this.ClientSize.Height);
            botRect = new Rectangle(0, this.ClientSize.Height - marg.Bottom, this.ClientSize.Width, marg.Bottom);

            // Fill Rectangles
            g.FillRectangle(blackBrush, topRect);
            g.FillRectangle(blackBrush, lefRect);
            g.FillRectangle(blackBrush, rigRect);
            g.FillRectangle(blackBrush, botRect);
        }

        GraphicsPath font = new GraphicsPath();
        SolidBrush brush = new SolidBrush(color);

        AddText(font, 24, 50, 20, Title);
        AddText(font, 16, 50, 70, Description);

        g.SmoothingMode = SmoothingMode.HighQuality;
        g.FillPath(brush, font);

        blackBrush.Dispose();
    }

    private void AddText(GraphicsPath font, float size, int x, int y, string text)
    {
        font.AddString(text, new FontFamily("Segoe UI"), (int)FontStyle.Regular, size, new Point(x, y), StringFormat.GenericDefault);
    }

    private void button1_Click(object sender, EventArgs e)
    {
      this.Paint -= new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);
      this.RecreateHandle(); //needed if changing on the fly
      this.FitGlass();
    }
        
    protected override void WndProc(ref Message m)
    {      
      base.WndProc(ref m);

      if (m.Msg == VistaApi.WM_NCHITTEST // if this is a click
        && m.Result.ToInt32() == VistaApi.HTCLIENT // ...and it is on the client
        && this.IsOnGlass(m.LParam.ToInt32())) // ...and specifically in the glass area
      {
        m.Result = new IntPtr(VistaApi.HTCAPTION); // lie and say they clicked on the title bar
      }
    }

    private bool IsOnGlass(int lParam)
    {
      // sanity check
      if (!this.IsGlassEnabled())
      {
        return false;
      }

      // get screen coordinates
      int x = (lParam << 16) >> 16; // lo order word
      int y =  lParam        >> 16; // hi order word

      // translate screen coordinates to client area
      Point p = this.PointToClient(new Point(x, y));

      // work out if point clicked is on glass
      if (topRect.Contains(p) || lefRect.Contains(p) || rigRect.Contains(p) || botRect.Contains(p))
      {
        return true;
      }

      return false;
    }

    private void buttonExit_Click(object sender, EventArgs e)
    {
        Application.Exit();
    }

    private void FormFlickr_Load(object sender, EventArgs e)
    {
        //var flickr = FlickrFactory.GetInstance();

        //if (flickr.IsAuthenticated)
        //{
            ChooseSource chooseSource = new ChooseSource();
            ChangePage(chooseSource, false);
        //}
        //else
        //{
        //    MessageBox.Show("You need to authenticate...");

        //    string frob = flickr.AuthGetFrob();
        //    string url = flickr.AuthCalcUrl(frob, FlickrNet.AuthLevel.Read);
        //    System.Diagnostics.Process.Start(url);
        //}

            ValidateAuthorization();
    }


    private void ValidateAuthorization()
    {
        if (PhotoService.Instance.HasToken)
        {
            try
            {
                var auth = PhotoService.Instance.Authorize();

                linkLabelLogin.Text = "Logout";
                labelStatus.Text = string.Format("You are authenticated as {0}.", auth.User.Username);
            }
            catch (FlickrException ex)
            {
                linkLabelLogin.Text = "Login";
                labelStatus.Text = "You are not authenticated.";

                if (ex.Message.Contains("98")) // Invalid Auth Token
                {
                    MessageBox.Show("Your authorization has expired or been removed. Please go through the authorization steps for Flickr2Mesh.", "Authorization Failure", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }
        else
        {
            linkLabelLogin.Text = "Login";
            labelStatus.Text = "You are not authenticated.";
        }
    }

    private void ChangePage(UserControl control, bool refresh)
    {
        control.Dock = DockStyle.Fill;

        _navigationIndex++;

        if (_navigationIndex > 0)
        {
            navigationControlLeft.Enabled = true;
        }

        if (_navigationControls.Count > _navigationIndex)
        {
            if (_navigationControls[_navigationIndex].GetType() == control.GetType())
            {
                control = null;
                ChangePage(_navigationIndex, refresh);
                return;
            }
            else
            {
                int index = _navigationIndex;
                _navigationControls.RemoveRange(index, _navigationControls.Count - index);
            }
        }

        IWizardPage wizard = (IWizardPage)control;
        Title = wizard.Title;
        Description = wizard.Description;
        panelPicture.BackgroundImage = wizard.Icon;
        //wizard.SearchOptions = SearchOptions;
        wizard.Next += new NextDelegate(FormFlickr_Next);

        panelContainer.Controls.Clear();
        panelContainer.Controls.Add(control);
        _navigationControls.Add(control);
        
        Invalidate();
    }

    private void ChangePage(int index, bool refresh)
    {
        Control control = _navigationControls[index];

        panelContainer.Controls.Clear();
        panelContainer.Controls.Add(control);

        IWizardPage wizard = (IWizardPage)control;
        Title = wizard.Title;
        Description = wizard.Description;
        panelPicture.BackgroundImage = wizard.Icon;
        /*wizard.SearchOptions = SearchOptions;*/

        if (refresh)
        {
            wizard.ReloadPage();

            int pageNumber = index + 1;

            // Check if there is more navigation controls than the next page
            if (_navigationControls.Count > pageNumber)
            {
                // We found additional navigation controls, we should remove them to
                // loose state.
                _navigationControls.RemoveRange(pageNumber, _navigationControls.Count - pageNumber);
            }

            navigationControlRight.Enabled = false;
        }

        Invalidate();
    }

    void FormFlickr_Next(UserControl source, bool refresh)
    {
        if (source is ChooseSource)
        {
            switch (Services.PhotoService.Instance.SearchOptions.Source)
            {
                case SourceOptions.User:
                    UserOptions ctrl = new UserOptions();
                    ChangePage(ctrl, refresh);
                    break;
                case SourceOptions.Group:
                    GroupOptions ctrl2 = new GroupOptions();
                    ChangePage(ctrl2, refresh);
                    break;
                case SourceOptions.Search:
                    EveryoneOptions ctrl3 = new EveryoneOptions();
                    ChangePage(ctrl3, refresh);
                    break;
            }
        }
        else if (source is DownloadPreview)
        {
            DownloadOptions options = new DownloadOptions();
            ChangePage(options, refresh);
        }
        else if (source is DownloadOptions)
        {
            DownloadProgress progress = new DownloadProgress();
            ChangePage(progress, refresh);
            progress.StartDownload();
            EnableThumbnailToolbarButtons();
        }
        else // one of the 3 options
        {
            DownloadPreview preview = new DownloadPreview();
            ChangePage(preview, refresh);
            preview.LoadPreview();
        }
    }

    private void NavigateBack()
    {
        if (_navigationIndex == 0)
            return;

        _navigationIndex--;

        if (_navigationIndex == 0)
            navigationControlLeft.Enabled = false;

        if (_navigationControls.Count > 1)
            navigationControlRight.Enabled = true;

        ChangePage(_navigationIndex, false);
    }

    private void NavigateForward()
    {
        if (_navigationControls.Count == _navigationIndex + 1)
            return;

        _navigationIndex++;

        if (_navigationIndex > 0)
        {
            navigationControlLeft.Enabled = true;
        }

        if (_navigationControls.Count == _navigationIndex + 1)
        {
            navigationControlRight.Enabled = false;
        }

        ChangePage(_navigationIndex, false);
    }

    private void panelButton_Click(object sender, EventArgs e)
    {
        NavigateBack();
    }

    private void panelForward_Paint(object sender, PaintEventArgs e)
    {
        NavigateForward();
    }

    private void button1_Click_1(object sender, EventArgs e)
    {
        NavigateForward();
    }

    private void linkAbout_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
    {
        About frm = new About();
        frm.ShowDialog();
    }

    private void linkLabelLogin_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
    {
        if (PhotoService.Instance.IsAuthenticated)
        {
            PhotoService.Instance.ResetAuthentication();
        }
        else
        {
            FlickrAuthForm frm = new FlickrAuthForm();
            frm.ShowDialog();
        }

        ValidateAuthorization();
    }
  }
}